@@ -56,61 +56,70 @@ enum
 
 static GParamSpec *obj_props[PROP_LAST];
 
-static GtkTextBuffer *
-build_eula_text_buffer_pango_markup (GFile   *file,
-                                     GError **error_out)
+typedef enum {
+  TEXT,
+  MARKUP,
+  SKIP,
+} FileType;
+
+static FileType
+get_file_type (GFile *file)
 {
-  GtkTextBuffer *buffer = NULL;
-  gchar *contents;
-  gsize length;
-  GError *error = NULL;
-  PangoAttrList *attrlist;
-  gchar *text;
-  GtkTextIter iter;
+  gchar *path, *last_dot;
+  FileType type;
 
-  if (!g_file_load_contents (file, NULL, &contents, &length, NULL, &error))
-    goto error_out;
+  path = g_file_get_path (file);
+  last_dot = strrchr (path, '.');
 
-  if (!pango_parse_markup (contents, length, 0, &attrlist, &text, NULL, &error))
-    goto error_out;
+  if (strcmp(last_dot, ".txt") == 0)
+    type = TEXT;
+  else if (strcmp (last_dot, ".xml") == 0)
+    type = MARKUP;
+  else
+    type = SKIP;
 
-  g_free (contents);
-
-  buffer = gtk_text_buffer_new (NULL);
-
-  gtk_text_buffer_get_end_iter (buffer, &iter);
-  text_buffer_insert_pango_text (buffer, &iter, attrlist, text);
-
-  return buffer;
-
- error_out:
-  g_propagate_error (error_out, error);
-  return NULL;
+  g_free (path);
+  return type;
 }
 
 static GtkTextBuffer *
-build_eula_text_buffer_plain_text (GFile   *file,
-                                   GError **error_out)
+build_eula_text_buffer (GFile     *file,
+                        GError   **error_out)
 {
   GtkTextBuffer *buffer = NULL;
   GtkTextIter start, end;
   GError *error = NULL;
   GInputStream *input_stream = NULL;
+  FileType type = get_file_type (file);
+
+  if (type == SKIP)
+    return NULL;
 
   input_stream = G_INPUT_STREAM (g_file_read (file, NULL, &error));
   if (input_stream == NULL)
     goto error_out;
 
   buffer = gtk_text_buffer_new (NULL);
-  splice_buffer (input_stream, buffer, &error);
-  if (error != NULL)
-    goto error_out;
 
-  /* monospace the text */
-  gtk_text_buffer_create_tag (buffer, "monospace", "family", "monospace", NULL);
-  gtk_text_buffer_get_start_iter (buffer, &start);
-  gtk_text_buffer_get_end_iter (buffer, &end);
-  gtk_text_buffer_apply_tag_by_name (buffer, "monospace", &start, &end);
+  switch (type) {
+  case TEXT:
+    if (!splice_buffer_text (input_stream, buffer, &error))
+      goto error_out;
+
+    /* monospace the text */
+    gtk_text_buffer_create_tag (buffer, "monospace", "family", "monospace", NULL);
+    gtk_text_buffer_get_start_iter (buffer, &start);
+    gtk_text_buffer_get_end_iter (buffer, &end);
+    gtk_text_buffer_apply_tag_by_name (buffer, "monospace", &start, &end);
+    break;
+  case MARKUP:
+    if (!splice_buffer_markup (input_stream, buffer, &error))
+      goto error_out;
+    break;
+  default:
+    g_assert_not_reached ();
+    break;
+  }
 
   return buffer;
 
@@ -126,18 +135,9 @@ build_eula_text_view (GFile *eula)
 {
   GtkWidget *widget = NULL;
   GtkTextBuffer *buffer;
-  gchar *path, *last_dot;
   GError *error = NULL;
 
-  path = g_file_get_path (eula);
-  last_dot = strrchr (path, '.');
-
-  if (strcmp(last_dot, ".txt") == 0)
-    buffer = build_eula_text_buffer_plain_text (eula, &error);
-  else if (strcmp (last_dot, ".xml") == 0)
-    buffer = build_eula_text_buffer_pango_markup (eula, &error);
-  else
-    goto out;
+  buffer = build_eula_text_buffer (eula, &error);
 
   if (buffer == NULL)
     goto out;
@@ -152,7 +152,6 @@ build_eula_text_view (GFile *eula)
     g_error_free (error);
   }
 
-  g_free (path);
   return widget;
 }
 
@@ -215,6 +214,13 @@ get_config (GFile    *eula,
   g_key_file_unref (config);
 }
 
+static GtkBuilder *
+gis_eula_page_get_builder (GisPage *page)
+{
+  /* Prevent warnings because of NULL page IDs */
+  return NULL;
+}
+
 static void
 gis_eula_page_constructed (GObject *object)
 {
@@ -287,7 +293,7 @@ gis_eula_page_constructed (GObject *object)
   gis_page_set_title (GIS_PAGE (page), _("License Agreements"));
   sync_page_complete (page);
 
-  gtk_widget_show (GTK_WIDGET (page));
+  gtk_widget_show_all (GTK_WIDGET (page));
 }
 
 
@@ -343,8 +349,10 @@ gis_eula_page_dispose (GObject *object)
 static void
 gis_eula_page_class_init (GisEulaPageClass *klass)
 {
+  GisPageClass *page_klass = GIS_PAGE_CLASS (klass);
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+  page_klass->get_builder = gis_eula_page_get_builder;
   object_class->get_property = gis_eula_page_get_property;
   object_class->set_property = gis_eula_page_set_property;
   object_class->constructed = gis_eula_page_constructed;
@@ -355,6 +363,8 @@ gis_eula_page_class_init (GisEulaPageClass *klass)
                          G_TYPE_FILE,
                          G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
 
+  g_object_class_install_properties (object_class, PROP_LAST, obj_props);
+
   g_type_class_add_private (object_class, sizeof(GisEulaPagePrivate));
 }
 
@@ -390,7 +400,7 @@ gis_prepare_eula_pages (GisDriver *driver)
     goto out;
 
   while ((info = g_file_enumerator_next_file (enumerator, NULL, &error)) != NULL) {
-    GFile *eula = g_file_get_child (eulas_dir, g_file_info_get_name (info));
+    GFile *eula = g_file_enumerator_get_child (enumerator, info);
     gis_driver_add_page (driver,
                          g_object_new (GIS_TYPE_EULA_PAGE,
                                        "driver", driver,
